@@ -1,4 +1,3 @@
-# coding=utf-8
 #
 # MIT License
 #
@@ -23,8 +22,9 @@
 # SOFTWARE.
 #
 
-from ._kuflow_rest_client import KuFlowRestClient
-from ._generated._serialization import Deserializer, Serializer, Model
+from ._generated._serialization import Deserializer, Model, Serializer
+from ._kuflow_rest_client import KuBotTokenCredential, KuFlowRestClient
 
-__all__ = ["Deserializer", "KuFlowRestClient", "Model", "Serializer"]
-__version__ = "0.4.2"
+
+__all__ = ["Deserializer", "KuBotTokenCredential", "KuFlowRestClient", "Model", "Serializer"]
+__version__ = "2.1.0"
