@@ -1,4 +1,3 @@
-# coding=utf-8
 #
 # MIT License
 #
@@ -23,21 +22,29 @@
 # SOFTWARE.
 #
 
-
+import base64
 import platform
 import sys
-import base64
-
 from typing import Any, Optional
-from azure.core.credentials import AccessToken
+
+from azure.core.credentials import AccessToken, TokenCredential
 from azure.core.pipeline.policies import SansIOHTTPPolicy
 
-from ._generated import VERSION, KuFlowRestClient as KuFlowRestClientGenerated
-
-from .operations import AuthenticationOperations, PrincipalOperations, ProcessOperations, TaskOperations
+from ._generated import VERSION
+from ._generated import KuFlowRestClient as KuFlowRestClientGenerated
+from .operations import (
+    AuthenticationOperations,
+    PrincipalOperations,
+    ProcessItemOperations,
+    ProcessOperations,
+    RobotOperations,
+    TenantOperations,
+    TenantUserOperations,
+    WorkerOperations,
+)
 
 
-class KuFlowClientTokenCredential:
+class ClientSecretCredential:
     def __init__(
         self,
         client_id: str,
@@ -46,18 +53,28 @@ class KuFlowClientTokenCredential:
         self.client_id = client_id
         self.client_secret = client_secret
 
-        self.token = base64.b64encode("{}:{}".format(client_id, client_secret).encode("utf-8")).decode("utf-8")
+        self.token = base64.b64encode(f"{client_id}:{client_secret}".encode()).decode("utf-8")
 
     def get_token(
-        self, *scopes: str, claims: Optional[str] = None, tenant_id: Optional[str] = None, **kwargs: Any
+        self,
+        *scopes: str,
+        claims: Optional[str] = None,
+        tenant_id: Optional[str] = None,
+        enable_cae: bool = False,
+        **kwargs: Any,
     ) -> AccessToken:
         """Request an access token for `scopes`.
 
-        :param str scopes: The type of access needed.
-
-        :keyword str claims: Additional claims required in the token, such as those returned in a resource
-            provider's claims challenge following an authorization failure.
-        :keyword str tenant_id: Optional tenant to include in the token request.
+        :param scopes: The type of access needed.
+        :type scopes: str
+        :keyword claims: Additional claims required in the token, such as those returned in a resource provider's
+                         claims challenge following an authorization failure.
+        :type claims: Optional[str]
+        :keyword tenant_id: Optional tenant to include in the token request.
+        :type tenant_id: Optional[str]
+        :keyword bool enable_cae: Indicates whether to enable Continuous Access Evaluation (CAE) for the requested
+                                  token. Defaults to False.
+        :type enable_cae: bool
 
         :rtype: AccessToken
         :return: An AccessToken instance containing the token string and its expiration time in Unix time.
@@ -65,8 +82,30 @@ class KuFlowClientTokenCredential:
         return AccessToken(token=self.token, expires_on=sys.maxsize)
 
 
+class KuBotTokenCredential(TokenCredential):
+    def __init__(self, token: str, expires_on: int) -> None:
+        self._cached_token = token
+        self._expires_on = expires_on
+
+    def get_token(
+        self,
+        *scopes: str,
+        claims: Optional[str] = None,
+        tenant_id: Optional[str] = None,
+        enable_cae: bool = False,
+        **kwargs: Any,
+    ) -> AccessToken:
+        return AccessToken(token=self._cached_token, expires_on=self._expires_on)
+
+    def __hash__(self):
+        return hash(self._cached_token)
+
+    def __eq__(self, other):
+        return self._cached_token == other._cached_token
+
+
 class AllowHttpPolicy(SansIOHTTPPolicy):
-    """A simple policy that allows http requestes adding "enforce_https" to the request context."""
+    """A simple policy that allows http requests adding "enforce_https" to the request context."""
 
     def on_request(self, request):
         """Updates with the given request id before sending the request to the next policy.
@@ -79,7 +118,7 @@ class AllowHttpPolicy(SansIOHTTPPolicy):
 
 
 class KuFlowRestClient:  # pylint: disable=client-accepts-api-version-keyword
-    API_VERSION = "v2022-10-08"
+    API_VERSION = "v2024-06-14"
 
     """Introduction
     ============
@@ -133,33 +172,38 @@ class KuFlowRestClient:  # pylint: disable=client-accepts-api-version-keyword
     choice automatically.
 
     :ivar authentication: AuthenticationOperations operations
-    :vartype authentication: kuflow.rest.client.operations.AuthenticationOperations
+    :type authentication: kuflow.rest.client.operations.AuthenticationOperations
     :ivar principal: PrincipalOperations operations
-    :vartype principal: kuflow.rest.client.operations.PrincipalOperations
+    :type principal: kuflow.rest.client.operations.PrincipalOperations
     :ivar process: ProcessOperations operations
-    :vartype process: kuflow.rest.client.operations.ProcessOperations
+    :type process: kuflow.rest.client.operations.ProcessOperations
     :ivar task: TaskOperations operations
-    :vartype task: kuflow.rest.client.operations.TaskOperations
-    :param client_id: Client id used to connect to KuFlow. Required.
-    :paramtype client_id: str
-    :param client_secret: Client secret used to connect to KuFlow. Required.
-    :paramtype client_secret: str
-    :keyword endpoint: Service URL. Default value is "https://api.kuflow.com/v2022-10-08".
-    :paramtype endpoint: str
+    :type task: kuflow.rest.client.operations.TaskOperations
+    :ivar worker: WorkerOperations operations
+    :type worker: kuflow.rest.client.operations.WorkerOperations
+    :param client_id: Client id used to connect to KuFlow. Required if credential is provided.
+    :type client_id: str
+    :param client_secret: Client secret used to connect to KuFlow. Required if credential is provided..
+    :type client_secret: str
+    :param credential: Client id used to connect to KuFlow. Required if no client_id and client_secret are provided.
+    :type credential: TokenCredential
+    :keyword endpoint: Service URL. Default value is "https://api.kuflow.com/v2024-06-14".
+    :type endpoint: str
     :keyword allow_insecure_connection: Allow non HTTPS endpoints. Default False.
-    :paramtype allow_insecure_connection: bool
+    :type allow_insecure_connection: bool
     """
 
     def __init__(
         self,
-        client_id: str,
-        client_secret: str,
+        client_id: Optional[str] = None,
+        client_secret: Optional[str] = None,
+        credential: Optional[TokenCredential] = None,
         endpoint: Optional[str] = None,
         allow_insecure_connection: Optional[bool] = None,
         **kwargs: Any,
     ) -> None:
         if endpoint is None:
-            endpoint = "https://api.kuflow.com/v2022-10-08"
+            endpoint = "https://api.kuflow.com/v2024-06-14"
 
         per_call_policies = []
         if allow_insecure_connection:
@@ -168,21 +212,32 @@ class KuFlowRestClient:  # pylint: disable=client-accepts-api-version-keyword
         if not endpoint.endswith("/" + KuFlowRestClient.API_VERSION):
             endpoint = endpoint + "/" + KuFlowRestClient.API_VERSION
 
+        if client_id is not None and client_secret is not None:
+            credential = ClientSecretCredential(client_id=client_id, client_secret=client_secret)
+
+        if credential is None:
+            raise Exception("client_id/client_secrets or credential is required")
+
+        python_version = platform.python_version()
+        platform_id = platform.platform()
+
         self._kuflow_client = KuFlowRestClientGenerated(
-            credential=KuFlowClientTokenCredential(client_id=client_id, client_secret=client_secret),  # type: ignore
+            credential=credential,
             endpoint=endpoint,
             api_version=kwargs.pop("api_version", VERSION),
-            credential_scopes="https://api.kuflow.com/v2022-10-08/.default",
+            credential_scopes="https://api.kuflow.com/v2024-06-14/.default",
             per_call_policies=per_call_policies,
-            base_user_agent="sdk-python-kuflow-rest/{} Python/{} ({})".format(
-                VERSION, platform.python_version(), platform.platform()
-            ),
+            base_user_agent=f"sdk-python-kuflow-rest/{VERSION} Python/{python_version} ({platform_id})",
         )
 
         self.authentication = AuthenticationOperations(self._kuflow_client)
         self.principal = PrincipalOperations(self._kuflow_client)
+        self.tenant_user = TenantUserOperations(self._kuflow_client)
         self.process = ProcessOperations(self._kuflow_client)
-        self.task = TaskOperations(self._kuflow_client)
+        self.process_item = ProcessItemOperations(self._kuflow_client)
+        self.worker = WorkerOperations(self._kuflow_client)
+        self.robot = RobotOperations(self._kuflow_client)
+        self.tenant = TenantOperations(self._kuflow_client)
 
     def __enter__(self):
         self._kuflow_client.__enter__()  # pylint:disable=no-member
