@@ -1,4 +1,3 @@
-# coding=utf-8
 #
 # MIT License
 #
@@ -26,7 +25,21 @@
 
 from ._authentication_operations import AuthenticationOperations
 from ._principal_operations import PrincipalOperations
+from ._process_item_operations import ProcessItemOperations
 from ._process_operations import ProcessOperations
-from ._task_operations import TaskOperations
+from ._robot_operations import RobotOperations
+from ._tenant_operations import TenantOperations
+from ._tenant_user_operations import TenantUserOperations
+from ._worker_operations import WorkerOperations
 
-__all__ = ["AuthenticationOperations", "PrincipalOperations", "ProcessOperations", "TaskOperations"]
+
+__all__ = [
+    "AuthenticationOperations",
+    "PrincipalOperations",
+    "ProcessItemOperations",
+    "ProcessOperations",
+    "RobotOperations",
+    "TenantOperations",
+    "TenantUserOperations",
+    "WorkerOperations",
+]
